﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Forms;
using HeuristicLab.Core;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Optimizer {
  internal partial class NewItemDialog : Form {
    private bool initialized;
    private List<IItem> items;

    private IItem item;
    public IItem Item {
      get { return item; }
    }

    public NewItemDialog() {
      initialized = false;
      items = new List<IItem>();
      item = null;
      InitializeComponent();
      SetEnabledStateOfControls();
    }

    private void NewItemDialog_Load(object sender, EventArgs e) {
      TaskScheduler context = TaskScheduler.FromCurrentSynchronizationContext();
      Task.Factory
        .StartNew((Action<object>)LoadItems, context)
        .ContinueWith(PrepareUI);
    }

    private void NewItemDialog_Shown(object sender, EventArgs e) {
      item = null;
    }

    private void LoadItems(object state) {
      TaskScheduler context = (TaskScheduler)state;
      if (!initialized) {
        Type[] creatableTypes = (from t in ApplicationManager.Manager.GetTypes(typeof(IItem))
                                 where CreatableAttribute.IsCreatable(t)
                                 select t).ToArray();
        var categories = from t in creatableTypes
                         orderby CreatableAttribute.GetCategory(t), ItemAttribute.GetName(t), ItemAttribute.GetVersion(t) ascending
                         group t by CreatableAttribute.GetCategory(t) into c
                         select c;

        itemsListView.SmallImageList = new ImageList();
        int count = 0;
        ListViewItem[] itemsArray = new ListViewItem[creatableTypes.Length];
        Image[] smallImagesArray = new Image[creatableTypes.Length];
        List<ListViewGroup> itemGroupsList = new List<ListViewGroup>();
        foreach (var category in categories) {
          ListViewGroup group = new ListViewGroup(category.Key);
          itemGroupsList.Add(group);
          foreach (var creatable in category) {
            IItem i = (IItem)Activator.CreateInstance(creatable);
            items.Add(i);
            ListViewItem item = new ListViewItem(new string[] { i.ItemName, i.ItemVersion.ToString(), i.ItemDescription }, group);
            item.ImageIndex = count;
            item.Tag = i;
            itemsArray[count] = item;
            smallImagesArray[count] = i.ItemImage;
            count++;
            Task.Factory.StartNew(() => {
              loadingProgressBar.Value = (int)Math.Round(loadingProgressBar.Minimum + (loadingProgressBar.Maximum - loadingProgressBar.Minimum) * (double)count / creatableTypes.Length);
            }, Task.Factory.CancellationToken, TaskCreationOptions.None, context);
          }
        }
        Task.Factory.StartNew(() => {
          itemsListView.SmallImageList.Images.AddRange(smallImagesArray);
          itemsListView.Groups.AddRange(itemGroupsList.ToArray());
          itemsListView.Items.AddRange(itemsArray);
          for (int i = 0; i < itemsListView.Columns.Count; i++)
            itemsListView.Columns[i].AutoResize(ColumnHeaderAutoResizeStyle.ColumnContent);
        }, Task.Factory.CancellationToken, TaskCreationOptions.None, context);
        initialized = true;
      }
    }

    private void PrepareUI(Task t) {
      SetEnabledStateOfControls();
      loadingPanel.Visible = false;
    }

    private void SetEnabledStateOfControls() {
      itemsListView.Enabled = initialized;
      okButton.Enabled = initialized;
      cancelButton.Enabled = initialized;
    }

    private void itemTypesListView_SelectedIndexChanged(object sender, EventArgs e) {
      okButton.Enabled = itemsListView.SelectedItems.Count == 1;
    }

    private void okButton_Click(object sender, EventArgs e) {
      if (itemsListView.SelectedItems.Count == 1) {
        item = (IItem)((IItem)itemsListView.SelectedItems[0].Tag).Clone();
        DialogResult = DialogResult.OK;
        Close();
      }
    }
    private void itemTypesListView_DoubleClick(object sender, EventArgs e) {
      if (itemsListView.SelectedItems.Count == 1) {
        item = (IItem)((IItem)itemsListView.SelectedItems[0].Tag).Clone();
        DialogResult = DialogResult.OK;
        Close();
      }
    }
  }
}
